@@ -619,11 +619,6 @@ get_file_age (GFile *file)
   return (guint64) (now - mtime);
 }
 
-static void
-no_progress_cb (OstreeAsyncProgress *progress, gpointer user_data)
-{
-}
-
 static guint64
 get_appstream_timestamp (FlatpakDir *dir,
                          const char *remote,
@@ -650,7 +645,6 @@ update_appstream (GPtrArray    *dirs,
                   GError      **error)
 {
   gboolean changed;
-  gboolean res;
   int i, j;
 
   g_return_val_if_fail (dirs != NULL, FALSE);
@@ -672,7 +666,6 @@ update_appstream (GPtrArray    *dirs,
           for (i = 0; remotes[i] != NULL; i++)
             {
               g_autoptr(GError) local_error = NULL;
-              g_autoptr(OstreeAsyncProgressFinish) progress = NULL;
               guint64 ts_file_age;
 
               ts_file_age = get_appstream_timestamp (dir, remotes[i], arch);
@@ -708,9 +701,8 @@ update_appstream (GPtrArray    *dirs,
                       g_print ("\n");
                     }
                 }
-              progress = ostree_async_progress_new_and_connect (no_progress_cb, NULL);
               if (!flatpak_dir_update_appstream (dir, remotes[i], arch, &changed,
-                                                 progress, cancellable, &local_error))
+                                                 NULL, cancellable, &local_error))
                 {
                   if (quiet)
                     g_debug ("%s: %s", _("Error updating"), local_error->message);
@@ -730,7 +722,6 @@ update_appstream (GPtrArray    *dirs,
 
           if (flatpak_dir_has_remote (dir, remote, NULL))
             {
-              g_autoptr(OstreeAsyncProgressFinish) progress = NULL;
               guint64 ts_file_age;
 
               found = TRUE;
@@ -744,10 +735,8 @@ update_appstream (GPtrArray    *dirs,
               else
                 g_debug ("%s:%s appstream age %" G_GUINT64_FORMAT " is greater than ttl %" G_GUINT64_FORMAT, remote, arch, ts_file_age, ttl);
 
-              progress = ostree_async_progress_new_and_connect (no_progress_cb, NULL);
-              res = flatpak_dir_update_appstream (dir, remote, arch, &changed,
-                                                  progress, cancellable, error);
-              if (!res)
+              if (!flatpak_dir_update_appstream (dir, remote, arch, &changed,
+                                                 NULL, cancellable, error))
                 return FALSE;
             }
         }
@@ -1349,23 +1338,33 @@ FlatpakRemoteState *
 get_remote_state (FlatpakDir   *dir,
                   const char   *remote,
                   gboolean      cached,
+                  gboolean      sideloaded,
                   GCancellable *cancellable,
                   GError      **error)
 {
   g_autoptr(GError) local_error = NULL;
-  FlatpakRemoteState *state;
+  FlatpakRemoteState *state = NULL;
 
-  state = flatpak_dir_get_remote_state (dir, remote, cached, cancellable, &local_error);
-  if (state == NULL && g_error_matches (local_error, FLATPAK_ERROR, FLATPAK_ERROR_NOT_CACHED))
+  if (sideloaded)
     {
-      g_clear_error (&local_error);
-      state = flatpak_dir_get_remote_state (dir, remote, FALSE, cancellable, &local_error);
+      state = flatpak_dir_get_remote_state_local_only (dir, remote, cancellable, error);
+      if (state == NULL)
+        return NULL;
     }
-
-  if (state == NULL)
+  else
     {
-      g_propagate_error (error, g_steal_pointer (&local_error));
-      return NULL;
+      state = flatpak_dir_get_remote_state (dir, remote, cached, cancellable, &local_error);
+      if (state == NULL && g_error_matches (local_error, FLATPAK_ERROR, FLATPAK_ERROR_NOT_CACHED))
+        {
+          g_clear_error (&local_error);
+          state = flatpak_dir_get_remote_state (dir, remote, FALSE, cancellable, &local_error);
+        }
+
+      if (state == NULL)
+        {
+          g_propagate_error (error, g_steal_pointer (&local_error));
+          return NULL;
+        }
     }
 
   return state;
