@@ -67,18 +67,18 @@ $FLATPAK info ${U} org.test.Hello > /dev/null
 $FLATPAK info ${U} org.test.Hello | grep test-repo > /dev/null
 $FLATPAK info ${U} org.test.Hello | grep $ID > /dev/null
 
-echo "ok install"
+ok "install"
 
 run org.test.Hello > hello_out
 assert_file_has_content hello_out '^Hello world, from a sandbox$'
 
-echo "ok hello"
+ok "hello"
 
 run_sh org.test.Platform cat /.flatpak-info >runtime-fpi
 assert_file_has_content runtime-fpi "[Runtime]"
 assert_file_has_content runtime-fpi "^runtime=runtime/org\.test\.Platform/$ARCH/stable$"
 
-echo "ok run a runtime"
+ok "run a runtime"
 
 if run org.test.Nonexistent 2> run-error-log; then
     assert_not_reached "Unexpectedly able to run non-existent runtime"
@@ -95,12 +95,12 @@ if run runtime/org.test.Nonexistent 2> run-error-log; then
 fi
 assert_file_has_content run-error-log "error: runtime/org\.test\.Nonexistent/\*unspecified\*/\*unspecified\* not installed"
 
-echo "ok error handling for invalid refs"
+ok "error handling for invalid refs"
 
 run_sh org.test.Hello cat /run/user/`id -u`/flatpak-info > fpi
 assert_file_has_content fpi '^name=org\.test\.Hello$'
 
-echo "ok flatpak-info"
+ok "flatpak-info"
 
 run_sh org.test.Hello readlink /proc/self/ns/net > unshared_net_ns
 ARGS="--share=network" run_sh org.test.Hello readlink /proc/self/ns/net > shared_net_ns
@@ -165,7 +165,7 @@ else
     echo "not testing \$HOME binding, \$HOME/.flatpak-tests/ does not exist" >&2
 fi
 
-echo "ok namespaces"
+ok "namespaces"
 
 test_overrides () {
     local dir="$1"
@@ -207,7 +207,7 @@ else
     echo "not testing \$HOME binding overrides, \$HOME/.flatpak-tests/ does not exist" >&2
 fi
 
-echo "ok overrides"
+ok "overrides"
 
 OLD_COMMIT=`${FLATPAK} ${U} info --show-commit org.test.Hello`
 
@@ -218,7 +218,7 @@ if [ x${USE_SYSTEMDIR-} != xyes ] ; then
     assert_streq "$OLD_COMMIT" "$ALSO_OLD_COMMIT"
 fi
 
-echo "ok null update"
+ok "null update"
 
 make_updated_app "" "" stable
 
@@ -231,7 +231,7 @@ assert_not_streq "$OLD_COMMIT" "$NEW_COMMIT"
 run org.test.Hello > hello_out
 assert_file_has_content hello_out '^Hello world, from a sandboxUPDATED$'
 
-echo "ok update"
+ok "update"
 
 ostree --repo=repos/test reset app/org.test.Hello/$ARCH/stable "$OLD_COMMIT"
 update_repo
@@ -244,7 +244,7 @@ NEW_NEW_COMMIT=`${FLATPAK} ${U} info --show-commit org.test.Hello`
 
 assert_streq "$NEW_COMMIT" "$NEW_NEW_COMMIT"
 
-echo "ok backwards update"
+ok "backwards update"
 
 make_updated_app "" "" stable UPDATED2
 
@@ -265,7 +265,7 @@ NEW_COMMIT=`${FLATPAK} ${U} info --show-commit org.test.Hello`
 
 assert_not_streq "$OLD_COMMIT" "$NEW_COMMIT"
 
-echo "ok install --or-update"
+ok "install --or-update"
 
 DIR=`mktemp -d`
 ${FLATPAK} build-init ${DIR} org.test.Split org.test.Platform org.test.Platform stable
@@ -348,7 +348,7 @@ assert_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/e/data
 assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/f
 assert_not_has_file $FL_DIR/app/org.test.Split/$ARCH/stable/active/files/nope
 
-echo "ok subpaths"
+ok "subpaths"
 
 VERSION=`cat "$test_builddir/package_version.txt"`
 
@@ -408,7 +408,7 @@ assert_file_has_content err_version.txt "needs a later flatpak version"
 assert_not_has_file $FL_DIR/app/org.test.CurrentVersion/$ARCH/stable/active/files/updated
 assert_has_file $FL_DIR/app/org.test.OldVersion/$ARCH/stable/active/files/updated
 
-echo "ok version checks"
+ok "version checks"
 
 rm -rf app
 flatpak build-init app org.test.Writable org.test.Platform org.test.Platform stable
@@ -416,7 +416,8 @@ mkdir -p app/files/a-dir
 chmod a+rwx app/files/a-dir
 flatpak build-finish --command=hello.sh app
 # Note: not --canonical-permissions
-ostree --repo=repos/test commit --owner-uid=0 --owner-gid=0  --no-xattrs  ${FL_GPGARGS} --branch=app/org.test.Writable/$ARCH/stable app
+${FLATPAK} build-export -vv --disable-sandbox --files=files repos/test app stable
+ostree --repo=repos/test commit  --keep-metadata=xa.metadata --owner-uid=0 --owner-gid=0  --no-xattrs  ${FL_GPGARGS} --branch=app/org.test.Writable/$ARCH/stable app
 update_repo
 
 # In the system-helper case this fails to install due to the permission canonicalization happening in the
@@ -426,7 +427,7 @@ if ${FLATPAK} ${U} install -y test-repo org.test.Writable; then
     assert_file_has_mode $FL_DIR/app/org.test.Writable/$ARCH/stable/active/files/a-dir 775
 fi
 
-echo "ok no world writable dir"
+ok "no world writable dir"
 
 rm -rf app
 flatpak build-init app org.test.Setuid org.test.Platform org.test.Platform stable
@@ -435,7 +436,8 @@ touch app/files/exe
 chmod u+s app/files/exe
 flatpak build-finish --command=hello.sh app
 # Note: not --canonical-permissions
-ostree --repo=repos/test commit --owner-uid=0 --owner-gid=0 --no-xattrs  ${FL_GPGARGS} --branch=app/org.test.Setuid/$ARCH/stable app
+${FLATPAK} build-export -vv --disable-sandbox --files=files repos/test app stable
+ostree -v --repo=repos/test commit --keep-metadata=xa.metadata --owner-uid=0 --owner-gid=0 --no-xattrs  ${FL_GPGARGS} --branch=app/org.test.Setuid/$ARCH/stable app
 update_repo
 
 if ${FLATPAK} ${U} install -y test-repo org.test.Setuid &> err2.txt; then
@@ -443,7 +445,7 @@ if ${FLATPAK} ${U} install -y test-repo org.test.Setuid &> err2.txt; then
 fi
 assert_file_has_content err2.txt [Ii]nvalid
 
-echo "ok no setuid"
+ok "no setuid"
 
 rm -rf app
 flatpak build-init app org.test.App org.test.Platform org.test.Platform stable
@@ -458,4 +460,4 @@ ${FLATPAK} ${U} info -m org.test.App > out
 
 assert_file_has_content out "^sdk=org\.test\.Sdk/$(flatpak --default-arch)/stable$"
 
-echo "ok --sdk option"
+ok "--sdk option"
